# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from re import compile
from os.path import getmtime

class AutoCompleteLaTeXSource:
	def __init__(self, wrapper):
		self.wrapper = wrapper
		self.filename = wrapper.get_filename()
		self.mtime = None
		self.packages = {}
	
	def get_break_chars(self):
		"""
		Return characters that break a word
		"""
		return "\n\t \\"
	
	def get_break_chars_retain(self):
		"""
		Return characters that break a word and must be retained
		"""
		return "\\"
	
	def query(self, prefix):
		if prefix[0] != "\\":
			return []
		
		prefix = prefix[1:]
		
		targets = self.wrapper.parser.get_targets()
		
		return [t for t in targets if str(t).startswith(prefix) and str(t) != prefix]
		
	def __file_changed(self):
		mtime = getmtime(self.filename)
		has_changed = (mtime != self.mtime)
		self.mtime = mtime
		return has_changed
	
	def __parse(self):
		packages = self.wrapper.parser.get_packages()
		
		curPackages = self.packages.keys()
		delPackages = list(set(curPackages).difference(packages))
		newPackages = list(set(packages).difference(curPackages))
		
		for p in delPackages:
			del self.packages[p]
		
		for p in newPackages:
			print "Adding LaTeX package %s" % p
			self.packages[p] = Package(p)
			
