# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Window, WINDOW_POPUP, ListStore, TreeView, CellRendererText, ScrolledWindow, POLICY_AUTOMATIC, \
		Frame, SHADOW_OUT

class AutoCompletePopup (Window):
	recent_target = None
	
	def __init__(self):
		Window.__init__(self, WINDOW_POPUP)
		
		self.model = ListStore(str, str, object)	# target, markup, Target object
		
		self.treeView = TreeView(self.model)
		
		self.treeView.insert_column_with_attributes(-1, "", CellRendererText(), markup=1)
		self.treeView.set_enable_search(False)
		self.treeView.set_headers_visible(False)
		
		self.scroll = ScrolledWindow()
		self.scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.scroll.add(self.treeView)
		#scroll.set_size_request(340, 160)
		
		frame = Frame()
		frame.set_shadow_type(SHADOW_OUT)
		frame.add(self.scroll)
		
		self.add(frame)
	
	def set_size(self, width, height):
		self.scroll.set_size_request(width, height)
	
	def get_recent_target(self):
		return self.recent_target
	
	def get_selection(self):
		listStore, listIter = self.treeView.get_selection().get_selected()
		self.recent_target = listStore.get_value(listIter, 2)
		return self.recent_target
	
	def move_cursor(self, delta):
		if delta == "H":
			self.treeView.set_cursor(0)
			return
		elif delta == "E":
			self.treeView.set_cursor(self.model.iter_n_children(None) - 1)
			return
			
		path = self.treeView.get_cursor()[0]
		if not path:
			return False
		index = path[0]
		maxIndex = self.model.iter_n_children(None)
		
		index += delta
		
		if index < 0:
			index = 0
		elif index >= maxIndex:
			index = maxIndex - 1
			
		self.treeView.set_cursor(index)
	
	def load_targets(self, targets):
		self.model.clear()
		for t in targets:
			self.model.append([str(t), t.get_markup(), t])
		
		self.treeView.set_cursor((0))
	
