# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK, \
	VBox, CheckButton, Alignment, Frame, SHADOW_NONE, Label, SpinButton, Adjustment, HBox, RadioButton, Notebook, STOCK_GO_UP, \
	STOCK_GO_DOWN, EXPAND, FILL
from gtk import ListStore, TreeView, CellRendererText, ScrolledWindow, POLICY_AUTOMATIC, SHADOW_IN, Alignment, Frame, SHADOW_NONE, \
	Table, Entry, Label, VButtonBox, BUTTONBOX_START, Button, STOCK_ADD, STOCK_DELETE, STOCK_EDIT
from md5 import new as create_key
from gtk import accelerator_parse, MessageDialog


from Settings import Settings, PLUGIN_PATH
from ProfileDialog import ProfileDialog
from common import Profile


class ConfigureDialog (Dialog):
	def __init__(self, settings):
		Dialog.__init__(self, _("Configure LaTeX Plugin"), None, DIALOG_MODAL|DIALOG_DESTROY_WITH_PARENT, (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_resizable(False)
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(5)
		
		self.profile_dialog = None
		#self.settings = Settings("%s/settings.xml" % PLUGIN_PATH)
		self.settings = settings
		
		## General page

		## Autocomplete / Autoclose
		
		lbl_input = Label()
		lbl_input.set_markup("<b>Input Assistance</b>")
		lbl_input.set_alignment(0, .5)

		self.check_autocomplete = CheckButton(_("Enable automatic completion"))
		self.check_autocomplete.set_active(bool(self.settings.setdefault("auto-completion-enabled", True)))
		self.check_autoclose = CheckButton(_("Enable automatic closing of environment commands"))
		self.check_autoclose.set_active(bool(self.settings.setdefault("auto-closing-enabled", True)))
		
		## DVI source specials
		
		lbl_dvi = Label()
		lbl_dvi.set_markup("<b>DVI Source Specials</b>")
		lbl_dvi.set_alignment(0, .5)
		
		self.check_dvi_search = CheckButton(_("Enable forward/inverse search in DVI documents via D-BUS"))
		self.check_dvi_search.set_active(bool(self.settings.setdefault("dvi-search-enabled", False)))
		
		## DPI
		
		lbl_dpi_h = Label()
		lbl_dpi_h.set_markup("<b>Unit Conversion</b>")
		lbl_dpi_h.set_alignment(0, .5)
		
		lbl_dpi = Label(_("Screen Resolution:"))
		lbl_dpi.set_alignment(0, .5)
		self.spin_dpi = SpinButton(Adjustment(float(self.settings.setdefault("dpi", 96)), 1, 1000, 1), 1)
		lbl_dpi_unit = Label("dpi")
		lbl_dpi_unit.set_alignment(0, .5)
		
		box_dpi = HBox()
		box_dpi.set_spacing(5)
		box_dpi.pack_start(lbl_dpi, False)
		box_dpi.pack_start(self.spin_dpi, False)
		box_dpi.pack_start(lbl_dpi_unit, False)
		
		## Warnings
		
#		lbl_warn = Label()
#		lbl_warn.set_markup("<b>Show Build Warnings</b>")
#		lbl_warn.set_alignment(0, .5)
#		
#		w = self.settings.setdefault("warnings", "off")
#		
#		self.rad_warn_off = RadioButton(None, _("Disable warnings"))
#		self.rad_warn_off.set_active(w == "off")
#		self.rad_warn_boxes = RadioButton(self.rad_warn_off, _("Overfull and underfull boxes"))
#		self.rad_warn_boxes.set_active(w == "boxes")
#		self.rad_warn_refs = RadioButton(self.rad_warn_off, _("Undefined or multiply defined references"))
#		self.rad_warn_refs.set_active(w == "refs")
#		self.rad_warn_all = RadioButton(self.rad_warn_off, _("Enable all warnings"))
#		self.rad_warn_all.set_active(w == "all")
#		
#		box_warn = VBox()
#		box_warn.pack_start(self.rad_warn_off, False, False)
#		box_warn.pack_start(self.rad_warn_boxes, False, False)
#		box_warn.pack_start(self.rad_warn_refs, False, False)
#		box_warn.pack_start(self.rad_warn_all, False, False)
		
		tbl_general = Table()
		tbl_general.set_border_width(10)
		tbl_general.set_row_spacings(5)
		tbl_general.set_col_spacings(5)
		tbl_general.attach(lbl_input, 0, 2, 0, 1)
		tbl_general.attach(self.check_autocomplete, 1, 2, 1, 2)
		tbl_general.attach(self.check_autoclose, 1, 2, 2, 3)
		tbl_general.attach(lbl_dvi, 0, 2, 3, 4)
		tbl_general.attach(self.check_dvi_search, 1, 2, 4, 5)
		tbl_general.attach(lbl_dpi_h, 0, 2, 5, 6)
		tbl_general.attach(box_dpi, 1, 2, 6, 7)
		tbl_general.set_col_spacing(0, 15)
		tbl_general.set_row_spacing(2, 15)
		tbl_general.set_row_spacing(4, 15)
		
		# Build Profiles
		self.store_profiles = ListStore(str, object) 		## markup, profile
		
		self.view_profiles = TreeView(self.store_profiles)
		self.view_profiles.set_headers_visible(False)
		self.view_profiles.insert_column_with_attributes(-1, "", CellRendererText(), markup=0)
		self.view_profiles.connect("row-activated", lambda *x: self.__configure_profile())
		
		scr_profiles = ScrolledWindow()
		scr_profiles.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scr_profiles.set_shadow_type(SHADOW_IN)
		scr_profiles.add(self.view_profiles)
		
		btn_add_profile = Button(stock=STOCK_ADD)
		btn_add_profile.connect("clicked", lambda x: self.__add_profile())
		
		btn_delete_profile = Button(stock=STOCK_DELETE)
		btn_delete_profile.connect("clicked", lambda x: self.__delete_profile())
		
		btn_edit_profile = Button(stock=STOCK_EDIT)
		btn_edit_profile.connect("clicked", lambda x: self.__configure_profile())
		
		buttons = VButtonBox()
		buttons.set_layout(BUTTONBOX_START)
		buttons.add(btn_add_profile)
		buttons.add(btn_edit_profile)
		buttons.add(btn_delete_profile)
		
		box_profiles = HBox()
		box_profiles.set_border_width(10)
		box_profiles.set_spacing(5)
		box_profiles.add(scr_profiles)
		box_profiles.pack_start(buttons, False)
		
		self.__load_profiles()
		#self.settings.monitor_profiles(lambda *x: self.__load_profiles())
		
		## Advanced Settings
		lbl_popup = Label()
		lbl_popup.set_markup("<b>Autocomplete Popup</b>")
		lbl_popup.set_alignment(0, .5)
		
		lbl_p_width = Label("Width:")
		lbl_p_width.set_alignment(0, .5)
		lbl_p_height = Label("Height:")
		lbl_p_height.set_alignment(0, .5)
		lbl_p_x = Label("X Offset:")
		lbl_p_x.set_alignment(0, .5)
		lbl_p_y = Label("Y Offset:")
		lbl_p_y.set_alignment(0, .5)
		
		self.spin_p_width = SpinButton(Adjustment(int(self.settings.setdefault("popup-width", 340)), 1, 1000, 1), 1)
		self.spin_p_height = SpinButton(Adjustment(int(self.settings.setdefault("popup-height", 160)), 1, 1000, 1), 1)
		self.spin_p_x = SpinButton(Adjustment(int(self.settings.setdefault("popup-x", 5)), -100, 100, 1), 1)
		self.spin_p_y = SpinButton(Adjustment(int(self.settings.setdefault("popup-y", 40)), -100, 100, 1), 1)
		
		tbl_advanced = Table()
		tbl_advanced.set_border_width(10)
		tbl_advanced.set_row_spacings(5)
		tbl_advanced.set_col_spacings(5)
		tbl_advanced.attach(lbl_popup, 0, 5, 0, 1, EXPAND|FILL, FILL)
		tbl_advanced.attach(lbl_p_width, 1, 2, 1, 2, EXPAND|FILL, FILL)
		tbl_advanced.attach(self.spin_p_width, 2, 3, 1, 2, EXPAND|FILL, FILL)
		tbl_advanced.attach(lbl_p_height, 3, 4, 1, 2, EXPAND|FILL, FILL)
		tbl_advanced.attach(self.spin_p_height, 4, 5, 1, 2, EXPAND|FILL, FILL)
		tbl_advanced.attach(lbl_p_x, 1, 2, 2, 3, EXPAND|FILL, FILL)
		tbl_advanced.attach(self.spin_p_x, 2, 3, 2, 3, EXPAND|FILL, FILL)
		tbl_advanced.attach(lbl_p_y, 3, 4, 2, 3, EXPAND|FILL, FILL)
		tbl_advanced.attach(self.spin_p_y, 4, 5, 2, 3, EXPAND|FILL, FILL)
		tbl_advanced.set_col_spacing(0, 15)
		tbl_advanced.set_col_spacing(2, 15)
		
		## Notebook
		notebook = Notebook()
		notebook.set_border_width(5)
		notebook.append_page(tbl_general, Label(_("General")))
		notebook.append_page(box_profiles, Label(_("Build Profiles")))
		notebook.append_page(tbl_advanced, Label(_("Advanced")))
		
		self.vbox.add(notebook)
		
		self.connect("response", self.__on_response)
		self.set_default_response(RESPONSE_OK)
		
		self.show_all()
	
	def __on_response(self, dialog, response):
		if response == RESPONSE_OK:
			self.settings["auto-completion-enabled"] = self.check_autocomplete.get_active()
			self.settings["auto-closing-enabled"] = self.check_autoclose.get_active()
			self.settings["dvi-search-enabled"] = self.check_dvi_search.get_active()
			self.settings["dpi"] = self.spin_dpi.get_value_as_int()
			self.settings["popup-width"] = self.spin_p_width.get_value_as_int()
			self.settings["popup-height"] = self.spin_p_height.get_value_as_int()
			self.settings["popup-x"] = self.spin_p_x.get_value_as_int()
			self.settings["popup-y"] = self.spin_p_y.get_value_as_int()
			
#			w = "off"
#			if self.rad_warn_boxes.get_active():
#				w = "boxes"
#			elif self.rad_warn_refs.get_active():
#				w = "refs"
#			elif self.rad_warn_all.get_active():
#				w = "all"
#			self.settings["warnings"] = w
			
		self.destroy()
	
	def __load_profiles(self):
		""" 
		Loads all profiles from Settings into the ListStore
		"""
		self.store_profiles.clear()
		for profile in self.settings.list_profiles():
			self.store_profiles.append([profile.get_markup(), profile])
	
	def __add_profile(self):
		"""
		Add a new profile
		"""
		if not self.profile_dialog:
			self.profile_dialog = ProfileDialog(self)
		
		self.profile_dialog.set_profile(None)
		
		if self.profile_dialog.run() == RESPONSE_OK:
			self.settings.save_profile(self.profile_dialog.get_profile())
			self.__load_profiles()
			
		self.profile_dialog.hide()
	
	def __configure_profile(self):
		""" 
		Configure the selected profile 
		"""
		list_store, list_iter = self.view_profiles.get_selection().get_selected()
		profile = list_store.get_value(list_iter, 1)
		
		if not self.profile_dialog:
			self.profile_dialog = ProfileDialog(self)
		
		self.profile_dialog.set_profile(profile)
		
		if self.profile_dialog.run() == RESPONSE_OK:
			p = self.profile_dialog.get_profile()
			self.settings.update_profile(p)
			self.__load_profiles()
		
		self.profile_dialog.hide()
	
	def __delete_profile(self):
		"""
		Delete the selected profile
		"""
		store, it = self.view_profiles.get_selection().get_selected()
		if it:
			profile = store.get_value(it, 1)
			self.settings.delete_profile(profile)
			store.remove(it)
			#self.__load_profiles()
			
