# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_DESTROY_WITH_PARENT, DIALOG_MODAL, STOCK_CANCEL, RESPONSE_CANCEL, \
	STOCK_OK, RESPONSE_OK, FileChooserDialog, FILE_CHOOSER_ACTION_OPEN, Image, Entry, Button, \
	STOCK_OPEN, HButtonBox, BUTTONBOX_END, VBox, Alignment, Frame, SHADOW_NONE, SpinButton, \
	Label, ToggleButton, RELIEF_NONE, Table, combo_box_new_text, CheckButton, FILL, FileChooserButton, \
	RESPONSE_ACCEPT, RadioButton, Adjustment
from gtk.gdk import pixbuf_new_from_file_at_size, pixbuf_new_from_file

from Settings import PLUGIN_PATH
from UnitComboBox import UnitComboBox, PERCENT, PIXEL
from common import ConditionalSource

class GraphicsDialog(Dialog):
	#
	# TODO: we need a preview for transformation
	#
	def __init__(self, instance):
		self.instance = instance
		self.source = None
		
		Dialog.__init__(self, _("Insert Graphics"), None, DIALOG_DESTROY_WITH_PARENT|DIALOG_MODAL, (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(10)
		
		## File
		self.file_dialog = FileChooserDialog(action=FILE_CHOOSER_ACTION_OPEN, buttons=(STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, RESPONSE_CANCEL))
		img_preview = Image()
		self.file_dialog.set_preview_widget(img_preview)
		self.file_dialog.set_local_only(True)
		self.file_dialog.connect("update-preview", self.__on_update_preview, img_preview)
		self.file_dialog.connect("response", self.__on_file_response)
		
		self.file_button = FileChooserButton(self.file_dialog)
		
		lbl_file = Label()
		lbl_file.set_markup(_("<b>_File</b>"))
		lbl_file.set_alignment(0, .5)
		lbl_file.set_mnemonic_widget(self.file_button)
		lbl_file.set_use_underline(True)
		
		#
		# size
		#
		self.spin_width = SpinButton()
		self.spin_width.set_activates_default(True)
		self.spin_width.set_range(1, 16000)
		self.spin_width.set_numeric(True)
		self.spin_width.set_digits(2)
		self.spin_width.set_increments(1, 10)
		
		self.spin_height = SpinButton()
		self.spin_height.set_activates_default(True)
		self.spin_height.set_range(1, 16000)
		self.spin_height.set_numeric(True)
		self.spin_height.set_digits(2)
		self.spin_height.set_increments(1, 10)
		
		self.lbl_width = Label(_("_Width:"))
		self.lbl_width.set_use_underline(True)
		self.lbl_width.set_mnemonic_widget(self.spin_width)
		self.lbl_width.set_alignment(0, .5)
		
		self.lbl_height = Label(_("_Height:"))
		self.lbl_height.set_use_underline(True)
		self.lbl_height.set_mnemonic_widget(self.spin_height)
		self.lbl_height.set_alignment(0, .5)
		
		self.cmb_unit = UnitComboBox(self.instance.settings["dpi"])
		self.cmb_unit.attach_spinbutton(self.spin_width)
		self.cmb_unit.attach_spinbutton(self.spin_height)
		self.cmb_unit.set_enable_percent(True)
		self.cmb_unit.set_unit(PERCENT)
		
		self.toggle_lock = ToggleButton(None)
		self.toggle_lock.set_relief(RELIEF_NONE)
		image = Image()
		image.set_from_file(PLUGIN_PATH + "/pixmaps/lock.png")
		self.toggle_lock.add(image)
		self.toggle_lock.connect("toggled", self.__lock_toggled)
		self.toggle_lock.set_active(True)
		
		self.lbl_original_size = Label()
		self.lbl_original_size.set_alignment(0, .5)
		
		self.lbl_size = Label()
		self.lbl_size.set_markup(_("<b>Size</b>"))
		self.lbl_size.set_alignment(0, .5)
		
		#
		# transform
		#
		self.__radio_no_transform = RadioButton(None, "No transformation")
		self.__radio_flip_h = RadioButton(self.__radio_no_transform, "Flip horizontal")
		self.__radio_flip_v = RadioButton(self.__radio_no_transform, "Flip vertical")
		self.__radio_rotate = RadioButton(self.__radio_no_transform, "Rotate:")
		self.__radio_rotate.connect("toggled", self.__radio_rotate_toggled)
		
		self.__spin_angle = SpinButton(Adjustment(90, -360, 360, 1))
		
		self.__lbl_transform = Label()
		self.__lbl_transform.set_markup(_("<b>Transform</b>"))
		self.__lbl_transform.set_alignment(0, .5)
		
		self.__lbl_angle_unit = Label("°")
		
		#
		# position (b|t|h|p)
		#
		self.check_pos_h = CheckButton(_("Where _Defined"))
		self.check_pos_t = CheckButton(_("_Top of Page"))
		self.check_pos_b = CheckButton(_("_Bottom of Page"))
		self.check_pos_p = CheckButton(_("Separate _Page"))
		
		for check_button in [self.check_pos_h, self.check_pos_t, self.check_pos_b, self.check_pos_p]:
			check_button.set_use_underline(True)
			check_button.set_active(True)
		
		self.check_center = CheckButton(_("Align _Centered"))
		self.check_center.set_use_underline(True)
		self.check_center.set_active(True)
		
		self.lbl_position = Label()
		self.lbl_position.set_markup(_("<b>Position</b>"))
		self.lbl_position.set_alignment(0, .5)
		
		## Caption
		self.entry_caption = Entry()
		self.entry_caption.set_activates_default(True)
		
		self.lbl_caption = Label()
		self.lbl_caption.set_markup(_("<b>_Caption</b>"))
		self.lbl_caption.set_alignment(0, .5)
		self.lbl_caption.set_use_underline(True)
		self.lbl_caption.set_mnemonic_widget(self.entry_caption)
		
		table = Table()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(lbl_file, 0, 5, 0, 1)
		table.attach(self.file_button, 1, 5, 1, 2)
		table.attach(self.lbl_original_size, 1, 5, 2, 3)
		table.attach(self.lbl_size, 0, 5, 3, 4)
		table.attach(self.lbl_width, 1, 2, 4, 5)
		table.attach(self.spin_width, 2, 3, 4, 5)
		table.attach(self.lbl_height, 1, 2, 5, 6)
		table.attach(self.spin_height, 2, 3, 5, 6)
		table.attach(self.toggle_lock, 3, 4, 4, 6)
		table.attach(self.cmb_unit, 4, 5, 5, 6)
		
#		table.attach(self.__lbl_transform, 0, 5, 6, 7)
#		table.attach(self.__radio_no_transform, 1, 5, 7, 8)
#		table.attach(self.__radio_flip_h, 1, 5, 8, 9)
#		table.attach(self.__radio_flip_v, 1, 5, 9, 10)
#		table.attach(self.__radio_rotate, 1, 2, 10, 11)
#		table.attach(self.__spin_angle, 2, 4, 10, 11)
#		table.attach(self.__lbl_angle_unit, 4, 5, 10, 11)
#		
#		table.attach(self.lbl_position, 0, 5, 11, 12)
#		table.attach(self.check_pos_h, 1, 3, 12, 13)
#		table.attach(self.check_pos_t, 3, 5, 12, 13)
#		table.attach(self.check_pos_b, 1, 3, 13, 14)
#		table.attach(self.check_pos_p, 3, 5, 13, 14)
#		table.attach(self.check_center, 1, 3, 14, 15)
#		table.attach(self.lbl_caption, 0, 5, 15, 16)
#		table.attach(self.entry_caption, 1, 5, 16, 17)
		
		table.attach(self.lbl_position, 0, 5, 6, 7)
		table.attach(self.check_pos_h, 1, 3, 7, 8)
		table.attach(self.check_pos_t, 3, 5, 7, 8)
		table.attach(self.check_pos_b, 1, 3, 8, 9)
		table.attach(self.check_pos_p, 3, 5, 8, 9)
		table.attach(self.check_center, 1, 3, 9, 10)
		table.attach(self.lbl_caption, 0, 5, 10, 11)
		table.attach(self.entry_caption, 1, 5, 11, 12)
		
		table.set_col_spacing(0, 15)
		table.set_row_spacing(2, 15)
		table.set_row_spacing(5, 15)
		table.set_row_spacing(10, 15)
#		table.set_row_spacing(14, 15)
		table.set_row_spacing(9, 15)
		
		self.vbox.add(table)
		
		self.set_default_response(RESPONSE_OK)
	
	def __radio_rotate_toggled(self, radio_rotate):
		v = self.__radio_rotate.get_active()
		self.__spin_angle.set_sensitive(v)
		self.__lbl_angle_unit.set_sensitive(v)
	
	def __lock_toggled(self, togglebutton):
		self.cmb_unit.set_keep_aspect_ratio(togglebutton.get_active())
	
	def __on_file_response(self, dialog, response):
		if response == RESPONSE_ACCEPT:
			filename = dialog.get_filename()
			
			self.cmb_unit.set_unit(PERCENT)
			self.spin_width.set_value(100)
			self.spin_height.set_value(100)
			
			try:
				# load the image to a pixbuf
				pixbuf = pixbuf_new_from_file(filename)
				
				# store original size and aspect ratio
				self.size = (pixbuf.get_width(), pixbuf.get_height())
				
				# show the image's original size
				self.lbl_original_size.set_markup("<small>%s x %s Pixels</small>" % (self.size[0], self.size[1]))
				
				self.cmb_unit.set_initial(self.spin_width, pixbuf.get_width(), PIXEL)
				self.cmb_unit.set_initial(self.spin_height, pixbuf.get_height(), PIXEL)
			except:
				pass
			
			self.__toggle_widgets(True)
			
	def run(self):
		self.__reset()
		self.show_all()
		return Dialog.run(self)
	
	def __reset(self):
		"""
		Reset the dialog
		"""
		self.source = None
		
		self.file_dialog.unselect_all()
		self.spin_width.set_value(1)
		self.spin_height.set_value(1)
		self.lbl_original_size.set_text("")
		self.entry_caption.set_text("")
		
		self.__toggle_widgets(False)
	
	def __toggle_widgets(self, setting):
		for widget in [self.lbl_size, self.lbl_width, self.spin_width, self.lbl_height, self.spin_height, 
				self.toggle_lock, self.cmb_unit, self.lbl_position, self.lbl_original_size, self.check_pos_h, 
				self.check_pos_t, self.check_pos_b, self.check_pos_p, self.check_center, self.lbl_caption,
				self.entry_caption, self.__lbl_transform, self.__lbl_angle_unit, self.__radio_no_transform,
				self.__radio_flip_h, self.__radio_flip_v, self.__radio_rotate, self.__spin_angle]:
			widget.set_sensitive(setting)
		
		if setting:
			rotate_enabled = self.__radio_rotate.get_active()
			self.__spin_angle.set_sensitive(rotate_enabled)
			self.__lbl_angle_unit.set_sensitive(rotate_enabled)
	
	def __on_update_preview(self, fileChooser, preview):
		""" Updates the FileChooser's preview image """
		filename = fileChooser.get_preview_filename()
		try:
			pixbuf = pixbuf_new_from_file_at_size(filename, 128, 128)
			preview.set_from_pixbuf(pixbuf)
			have_preview = True
		except:
			have_preview = False
		fileChooser.set_preview_widget_active(have_preview)
	
	def get_source(self):
		#
		# see this:
		# http://www.artofproblemsolving.com/LaTeX/AoPS_L_PictMan.php
		#
		if not self.source:
			filename = self.file_dialog.get_filename()
			caption = self.entry_caption.get_text()
			
			#
			# scale
			#
			unit = self.cmb_unit.get_active()
			x = self.spin_width.get_value()
			y = self.spin_height.get_value()
			
			options = []
			if unit == PERCENT:
				if x != 100 or y != 100:
					options.append("scale=%s" % round(x/100, 2))
			else:
				unit_s = ["","px","in","mm"][unit]
				options.append("width=%s%s" % (round(x, 2), unit_s))
				options.append("height=%s%s" % (round(y, 2), unit_s))
			
			if self.__radio_flip_v.get_active():
				options.append("angle=180")
			elif self.__radio_rotate.get_active():
				options.append("angle=%s" % self.__spin_angle.get_value_as_int())
			
			s = "\\includegraphics[%s]{%s}" % (",".join(options), filename)
			
			#
			# transform
			#
			if self.__radio_flip_h.get_active() or self.__radio_flip_v.get_active():
				s = "\\reflectbox{%s}" % s
			
			if self.check_center.get_active():
				s = "\\begin{center}\n\t\t%s\n\t\\end{center}" % s
			
			#
			# position
			#
			pos = ""
			if self.check_pos_h.get_active():
				pos += "h"
			if self.check_pos_t.get_active():
				pos += "t"
			if self.check_pos_b.get_active():
				pos += "b"
			if self.check_pos_p.get_active():
				pos += "p"
			if len(pos):
				pos = "[%s]" % pos
			
			s = "\\begin{figure}%s\n\t%s" % (pos, s)
			if len(caption):
				s = "%s\n\t\\caption{%s}" % (s, caption)
			s = "%s\n\\end{figure}" % s

			self.source = ConditionalSource(s, ["graphicx"])
			
		return self.source
		
