# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ActionGroup, STOCK_CONVERT, CheckMenuItem, MenuItem, Image, STOCK_INFO, \
		RESPONSE_OK, accelerator_parse, ACCEL_VISIBLE, AccelGroup
from gobject import timeout_add
from os import getcwd

from Logger import log
from DocumentWrapper import DocumentWrapper
from StructurePanel import StructurePanel
from SymbolsPanel import SymbolsPanel
from SnippetsPanel import SnippetsPanel
from MainToolbar import MainToolbar
from Builder import Builder
from Settings import Settings, PLUGIN_PATH
#from common import PLUGIN_PATH
from AutoCompletePopup import AutoCompletePopup
from BuildView import BuildView
from ConsoleView import ConsoleView
from QuickstartDialog import QuickstartDialog
from TableDialog import TableDialog
from GraphicsDialog import GraphicsDialog


#DBUS_BUS_NAME = 'org.gedit.LaTeXPlugin'
#DBUS_OBJECT_PATH = '/org/gedit/DBUSInterface'


class PluginInstance(object):
	def __init__(self, plugin, window, settings):
		self.plugin, self.window = plugin, window
        
		self.__init_wrappers()
		self.active_wrapper = None
		self.current_context_id = None
        
		self.settings = settings
		self.settings.connect("profile-saved", lambda *x: self.__update_profiles())
		self.settings.connect("profile-updated", lambda *x: self.__update_profiles())
		self.settings.connect("profile-deleted", lambda *x: self.__update_profiles())
		
		self.builder = Builder(self)
		self.__insert_widgets()
		self.save_action = self.ui_manager.get_action("/MenuBar/FileMenu/FileSaveMenu")
		self.auto_complete_popup = AutoCompletePopup()
		
		self.__signals_ids = []
		self.__signals_ids.append(self.window.connect("tab_added", self.__tab_added))
		self.__signals_ids.append(self.window.connect("tab_removed", self.__tab_removed))
		self.__signals_ids.append(self.window.connect("active_tab_changed", self.__active_tab_changed))
		
		self.__dlg_quickstart = None
		self.__dlg_table = None
		self.__dlg_graphics = None
	
	def __init_wrappers(self):
		"""
		If the plugin is loaded after gedit has started, then we must wrap all open
		documents.
		"""
		log(self, "__init_wrappers")
		
		self.document_wrappers = {}
		
		for view in self.window.get_views():
		    self.document_wrappers[view] = DocumentWrapper(self, view)
		    
		log(self, "__init_wrappers finished")
	
	def __tab_added(self, window, tab):
		"""
		A tabs was added
		Create a wrapper for its view
		"""
		log(self, "__tab_added")
		
		view = tab.get_view()
		
		#
		# When gedit is started, the signal "active_tab_changed" is emitted before "tab_added".
		# So there may already be a wrapper for this tab.
		#
		try:
			wrapper = self.document_wrappers[view]
		except KeyError:
			self.document_wrappers[view] = DocumentWrapper(self, view)
    
	def __tab_removed(self, window, tab):
		"""
		A tab was removed
		Remove its wrapper
		"""
		log(self, "__tab_removed")
		
		del self.document_wrappers[tab.get_view()]
	
	def __active_tab_changed(self, window, tab):
		log(self, "__active_tab_changed")
		
		#
		# When gedit is started, the signal "active_tab_changed" is emitted before "tab_added".
		# So there may be no wrapper for this tab, yet.
		#
		try:
			wrapper = self.document_wrappers[tab.get_view()]
		except KeyError:
			view = tab.get_view()
			wrapper = DocumentWrapper(self, view)
			self.document_wrappers[view] = wrapper
		
		self.structure_panel.set_structure_model(wrapper.get_structure_model())	## display its structure...
		self.structure_panel.set_labels_model(wrapper.get_labels_model())		## ...and labels/bibitems
		
		self.active_wrapper = wrapper
    
#	def __load_dbus_interface(self):
#		try:
#			import dbus.service
#			if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
#				import dbus.glib
#			
#			class DBUSInterface (dbus.service.Object):
#				"""
#				The DBUSInterface is listening for DVI inverse search commands and
#				scrolls the document to the appropriate line
#				"""
#				def __init__(self, instance, bus_name, object_path=DBUS_OBJECT_PATH):
#					dbus.service.Object.__init__(self, bus_name, object_path)
#					self.instance = instance
#
#				# This is for dbus inverse search
#				@dbus.service.method('org.gedit.LaTeXPluginIFace')
#				def inverse(self, filename, line):
#					print "Got inverse DVI search: " + filename + " " + str(line)
#					
#					self.instance.set_active_tab_by_filename(filename)
#					wrapper = self.instance.get_active_wrapper()
#					wrapper.jump_to_line(line)
#
#
#				# adding this makes things work---don't know why!
#				@dbus.service.method('org.gedit.LaTeXPluginIFace')
#				def dummy(self):
#					pass
#		
#			# DBUS interface
#			bus_name = dbus.service.BusName(DBUS_BUS_NAME, bus=dbus.SessionBus())
#			self.dbus_interface = DBUSInterface(self, bus_name)
#			
#		except ImportError:
#			print "Warning: failed to import DBUS bindings"
	
	def update(self):
#		active_view = self.window.get_active_view()
#		if not active_view:
#			return
#			
#		try:
#			## activate the DocumentWrapper connected to that GeditView
#			wrapper = self.document_wrappers[active_view]
#			wrapper.activate()
#		except KeyError:
#			## create a new DocumentWrapper
#			wrapper = DocumentWrapper(self, active_view)
#			self.document_wrappers[active_view] = wrapper
#		
#		if wrapper != self.active_wrapper:
#			## the active wrapper has changed...
#			self.structure_panel.set_structure_model(wrapper.get_structure_model())	## display its structure...
#			self.structure_panel.set_labels_model(wrapper.get_labels_model())		## ...and labels/bibitems
#		
#			self.active_wrapper = wrapper
		pass
		
	def deactivate(self):
		# disconnect signals
		for signal_id in self.__signals_ids:
		    self.window.disconnect(signal_id)
		
		self.__remove_widgets()
	
	def __pop_status_message(self):
		status_bar = self.window.get_statusbar()
		if self.current_context_id:
			status_bar.pop(self.current_context_id)
		self.current_context_id = None
		
		return False		## do not call again
	
	def push_status_message(self, message):
		"""
		Push a message to the window's status bar and start a timer that pops it again
		"""
		status_bar = self.window.get_statusbar()
		self.current_context_id = status_bar.get_context_id(message)
		status_bar.push(self.current_context_id, message)
	
		timeout_add(3000, self.__pop_status_message)
		
	def get_active_wrapper(self):
		return self.active_wrapper
	
	def save_active_document(self):
		self.save_action.activate()
	
	def set_active_tab_by_filename(self, filename):
		"""
		Activate the document tab with a specific filename
		"""
		if filename[0] != "/":
			filename = "%s/%s" % (getcwd(), filename)	## rubber returns relative filenames
		
		## check if the tab is already active
		if self.active_wrapper.get_filename() == filename:
			return
		
		## find the tab and activate it
		for wrapper in self.document_wrappers.values():
			if wrapper.get_filename() == filename:
				tab = gedit.tab_get_from_document(wrapper.get_document())
				self.window.set_active_tab(tab)
				return
		
		## try to open the document in gedit
		print self.window.create_tab_from_uri(filename, None, False, True)
		
		print "set_active_tab_by_filename(): %s, document not found" % filename
	
	def __insert_widgets(self):
		#
		# attach side panels
		#
		self.structure_panel = StructurePanel(self)
		self.symbols_panel = SymbolsPanel(self)
		self.snippets_panel = SnippetsPanel(self)
		
		img_outline = Image()
		img_outline.set_from_file(PLUGIN_PATH + "/pixmaps/outline.png")
		
		img_symbols = Image()
		img_symbols.set_from_file(PLUGIN_PATH + "/pixmaps/symbols.png")
		
		img_snippets = Image()
		img_snippets.set_from_file(PLUGIN_PATH + "/pixmaps/snippet.png")
		
		self.window.get_side_panel().add_item(self.structure_panel, _("Document Structure"), img_outline)
		self.window.get_side_panel().add_item(self.symbols_panel, _("Symbol Map"), img_symbols)
		self.window.get_side_panel().add_item(self.snippets_panel, _("Snippets"), img_snippets)
		
		#
		# attach bottom panels
		#
		self.build_view = BuildView()
		self.build_view.connect("line-selected", self.__line_selected)
		self.build_view.connect("abort-clicked", self.__builder_aborted)
		
		self.console_view = ConsoleView()
		
		self.window.get_bottom_panel().add_item(self.build_view, "Build", STOCK_CONVERT)
		self.window.get_bottom_panel().add_item(self.console_view, "Console Output", STOCK_INFO)
		
		#
		# insert menu items
		#
		self.ui_manager = self.window.get_ui_manager()
		self.action_group = ActionGroup("LaTeXPluginActions")
		self.action_group.add_actions([
				("Comment", None, _("Comment"), "<Ctrl><Alt>C", None, lambda x: self.get_active_wrapper().comment_selection()),
				("Uncomment", None, _("Uncomment"), "<Ctrl><Alt>U", None, lambda x: self.get_active_wrapper().comment_selection(uncomment=True)),
				
				("Latex", None, "LaTeX", None, None, None),
				
				("LatexQuickstart", None, "New LaTeX Document...", "<Ctrl><Alt>N", None, lambda x: self.run_quickstart_dialog()),
				("InsertTable", None, "Insert Table...", "<Ctrl><Alt>T", None, lambda x: self.run_table_dialog()),
				("InsertGraphics", None, "Insert Graphics...", "<Ctrl><Alt>G", None, lambda x: self.run_graphics_dialog()),
				("Build", STOCK_CONVERT, _("Build LaTeX Document"), None, None, None)
#				("BuildPDF", None, "PDF", "F5", _("Builds a PDF file from the document"), lambda x: self.builder.build_pdf()),
#				("BuildDVI", None, "DVI", "<Ctrl>F5", _("Builds a DVI file from the document"), lambda x: self.builder.build_dvi()),
#				("BuildPS", None, "PostScript", "<Shift><Ctrl>F5", _("Builds a PostScript file from the document"), lambda x: self.builder.build_ps()),
#				("BuildImage", None, _("Image") + "...", None, _("Builds an image from the document"), lambda x: self.builder.build_image())
		])
		
		
		
#		counter = 1
#		for profile in self.settings.list_profiles():
#			self.action_group.add_actions([(profile.name, None, "Build this document using the profile %s" % profile.name, "<Ctrl><Alt>%s" % counter, None, lambda x: self.build(profile))])
#			counter += 1
		
		self.ui_manager.insert_action_group(self.action_group, -1)
		self.ui_id = self.ui_manager.add_ui_from_file(PLUGIN_PATH + "/ui.xml")
		
		#
		# insert profiles
		#

#		build_menu = self.ui_manager.get_widget("/MenuBar/LatexMenu/BuildMenu").get_submenu()
#		
#		accel_group = AccelGroup()
#		build_menu.set_accel_group(accel_group)
#		
#		# 
#		# FIXME: only the first accel activates only the last item
#		#
#		
#		#counter = 1
#		for profile in self.settings.list_profiles():
#			item = MenuItem(profile.name)
#			item.connect("activate", lambda x: self.build(profile))
#
#			#key, mods = accelerator_parse("<Ctrl><Alt>%s" % counter)
#			if profile.accel:
#				key, mods = accelerator_parse(profile.accel)
#				item.add_accelerator("activate", accel_group, key, mods, ACCEL_VISIBLE)
#			
#			item.show()
#			build_menu.append(item)
#			
#			#counter += 1
#		
#		self.window.add_accel_group(accel_group)
		
		self.__merge_profiles()
		
		## hook the toolbar
		self.toolbar = MainToolbar(self)
		self.main_box = self.window.get_children()[0]
		self.main_box.pack_start(self.toolbar, False)
		self.main_box.reorder_child(self.toolbar, 2)
		
		## add a CheckMenuItem for the toolbar
		self.item_show_toolbar = CheckMenuItem(_("LaTeX Toolbar"))
		self.item_show_toolbar.set_active(True)
		self.ui_manager.get_widget("/MenuBar/ViewMenu").get_submenu().insert(self.item_show_toolbar, 2)
		self.item_show_toolbar.show()
		self.item_show_toolbar.connect("toggled", self.__on_toolbar_toggled)
	
	def __merge_profiles(self):
		"""
		Merge menu entries for the build profiles
		"""
		#
		# create XML definition
		#
		ui_top = """<ui>
				<menubar name="MenuBar">
					<placeholder name="ExtraMenu_1">
						<menu name="LatexMenu" action="Latex">
							<menu name="BuildMenu" action="Build">"""
					
		ui_bottom = """</menu>
							</menu>
						</placeholder>
					</menubar>
				</ui>"""
		
		s = ""
		actions = []
		for profile in self.settings.list_profiles():
			s += "<menuitem name=\"%s_Menu\" action=\"%s\"/>" % (profile.name, profile.name)
			actions.append(("%s" % profile.name, None, profile.name, profile.accel, None, self.__profile_activated))
		
		ui = ui_top + s + ui_bottom
		#
		# create actions
		#
		self.__group_profiles = ActionGroup("LaTeXPluginBuildActions")
		self.__group_profiles.add_actions(actions)
		
		self.ui_manager.insert_action_group(self.__group_profiles, -1)
		#
		# merge
		#
		self.__ui_profiles = self.ui_manager.add_ui_from_string(ui)
	
	def __unmerge_profiles(self):
		self.ui_manager.remove_ui(self.__ui_profiles)
		self.ui_manager.remove_action_group(self.__group_profiles)
		self.ui_manager.ensure_update()
	
	def __update_profiles(self):
		self.__unmerge_profiles()
		self.__merge_profiles()
	
	def __profile_activated(self, action):
		log(self, "__profile_activated : %s, %s" % (str(action), action.get_name()))
		
		self.build(self.settings.get_profile(action.get_name()))
	
	def __remove_widgets(self):
		## remove panels
		self.window.get_side_panel().remove_item(self.symbols_panel)
		self.window.get_side_panel().remove_item(self.structure_panel)
		
		#
		# remove menu items
		#
		self.__unmerge_profiles()
		
		self.ui_manager.remove_ui(self.ui_id)
		self.ui_manager.remove_action_group(self.action_group)
		self.ui_manager.ensure_update()
		
		## remove toolbar
		self.main_box.remove(self.toolbar)
		
		## destroy
		self.toolbar, self.symbols_panel, self.structure_panel = None, None, None
	
	def build(self, profile):
		#
		# TODO: this should be done by a BuildController
		#
		log(self, "build : " + profile.name)
		
		self.save_active_document()
		
		wrapper = self.get_active_wrapper()

		wrapper.reset_highlight()
		
		filename=None
		if wrapper.is_master():
			filename = wrapper.get_filename()
		else:
			filename = wrapper.get_master_filename()
		
		if not filename:
			return
		
		self.builder.set_filename(filename)
		self.builder.set_profile(profile)
		self.builder.run()
	
	def __line_selected(self, widget, uri, line):
		"""
		A widget like the BuildView or the StructureView signaled that a source
		line has been selected
		"""
		#
		# TODO: find the right tab by the URI
		#
		self.get_active_wrapper().jump_to_line(line)
	
	def __on_toolbar_toggled(self, checkmenuitem):
		if checkmenuitem.get_active():
			self.toolbar.show()
		else:
			self.toolbar.hide()
	
	def __activate_build_view(self):
		"""
		Show the bottom panel and select the tab containing the BuildView
		"""
		# TODO: ...
		pass
	
	def __builder_aborted(self, buildview):
		self.builder.abort()
	
	def run_quickstart_dialog(self):
		if not self.__dlg_quickstart:
			self.__dlg_quickstart = QuickstartDialog(self)
		
		if self.__dlg_quickstart.run() == RESPONSE_OK:
			wrapper = self.get_active_wrapper()
			wrapper.insert_source(self.__dlg_quickstart.get_source())
			wrapper.enable_highlighting()
		self.__dlg_quickstart.hide()
	
	def run_table_dialog(self):
		if not self.__dlg_table:
			self.__dlg_table = TableDialog()
		
		wrapper = self.get_active_wrapper()
		
		if self.__dlg_table.run(wrapper.get_environment()) == RESPONSE_OK:
			wrapper.insert_source(self.__dlg_table.get_source())
			
		self.__dlg_table.hide()
	
	def run_graphics_dialog(self):
		if not self.__dlg_graphics:
			self.__dlg_graphics = GraphicsDialog(self)
		
		if self.__dlg_graphics.run() == RESPONSE_OK:
			wrapper = self.get_active_wrapper()
			wrapper.insert_source(self.__dlg_graphics.get_source())
		self.__dlg_graphics.hide()
	