# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import TreeView, TreeViewColumn, CellRendererText, CellRendererPixbuf, ScrolledWindow, \
		POLICY_AUTOMATIC, SHADOW_IN, VPaned
from gtk.gdk import pixbuf_new_from_file

from common import LABEL, BIBITEM, PLUGIN_PATH

try:
	pb_label = pixbuf_new_from_file(PLUGIN_PATH + "/pixmaps/label.png")
	pb_bibitem = pixbuf_new_from_file(PLUGIN_PATH + "/pixmaps/bibitem.png")
except:
	pb_label, pb_bibitem = None, None

class StructurePanel (VPaned):
	#
	# TODO: Icons verwenden wie im BuildView
	#
	def __init__(self, instance):
		self.instance = instance
		VPaned.__init__(self)
		
		self.connect("button-release-event", self.__position_changed)
		
		#
		# structure
		#
		
#		self.__model = TreeStore(Pixbuf, str, object, str, str)		# icon, markup, Message object, line, file
#		
#		self.__view = TreeView(self.__model)
#		#self.__view.set_headers_visible(False)
#		
#		column = TreeViewColumn("Job")
#
#		pixbuf_renderer = CellRendererPixbuf()
#		column.pack_start(pixbuf_renderer, False)
#		column.add_attribute(pixbuf_renderer, "pixbuf", 0)
#		
#		text_renderer = CellRendererText()
#		column.pack_start(text_renderer, True)
#		column.add_attribute(text_renderer, "markup", 1)
#		
#		self.__view.append_column(column)
#		self.__view.append_column(TreeViewColumn("Line", CellRendererText(), text=3))
#		self.__view.append_column(TreeViewColumn("File", CellRendererText(), text=4))
		
		self.view_structure = TreeView()
		self.view_structure.set_headers_visible(False)
		
#		self.view_structure.append_column(TreeViewColumn(None, CellRendererText(), markup=0))

		column = TreeViewColumn()
		
		pixbuf_renderer = CellRendererPixbuf()
		column.pack_start(pixbuf_renderer, False)
		column.add_attribute(pixbuf_renderer, "pixbuf", 3)
		
		text_renderer = CellRendererText()
		column.pack_start(text_renderer, True)
		column.add_attribute(text_renderer, "markup", 0)
		
		self.view_structure.append_column(column)
		
		self.view_structure.connect("cursor-changed", self.__structure_cursor_changed)
		
		scr_structure = ScrolledWindow()
		scr_structure.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scr_structure.set_shadow_type(SHADOW_IN)
		scr_structure.add(self.view_structure)
		
		self.add1(scr_structure)
		
		self.view_labels = TreeView()
		self.view_labels.set_headers_visible(False)
		cell_pixbuf = CellRendererPixbuf()
		self.view_labels.insert_column_with_attributes(-1, "", cell_pixbuf)
		self.view_labels.get_column(0).set_cell_data_func(cell_pixbuf, self.__cell_data_function)
		self.view_labels.insert_column_with_attributes(-1, "", CellRendererText(), text=0)
		self.view_labels.connect("cursor-changed", self.__label_cursor_changed)
		
		scr_labels = ScrolledWindow()
		scr_labels.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scr_labels.set_shadow_type(SHADOW_IN)
		scr_labels.add(self.view_labels)
		
		self.add2(scr_labels)
		
		self.show_all()
		
		self.set_position(int(instance.settings.setdefault("structure-paned-position", -1)))
	
	def set_structure_model(self, model):
		self.view_structure.set_model(model)
		self.view_structure.expand_all()
	
	def set_labels_model(self, model):
		self.view_labels.set_model(model)
	
	def expand_structure(self):
		self.view_structure.expand_row("0", True)
	
	def __position_changed(self, paned, event):
		self.instance.settings["structure-paned-position"] = paned.get_position()
	
	def __cell_data_function(self, column, cell, model, it):
		t = model.get_value(it, 3)
		pixbuf = pb_label
		if t == BIBITEM:
			pixbuf = pb_bibitem
		cell.set_property("pixbuf", pixbuf)
	
	def __structure_cursor_changed(self, treeview):
		store, it = treeview.get_selection().get_selected()
		if not it: 
			return
			
		start, end = store.get_value(it, 1), store.get_value(it, 2)
		self.instance.get_active_wrapper().select_range(start, end)
	
	def __label_cursor_changed(self, treeview):
		store, it = treeview.get_selection().get_selected()
		if not it: 
			return
			
		start, end = store.get_value(it, 1), store.get_value(it, 2)
		self.instance.get_active_wrapper().select_range(start, end)
	
