# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, \
	RESPONSE_OK, FileChooserDialog, FILE_CHOOSER_ACTION_OPEN, STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, \
	RESPONSE_CANCEL, FileChooserButton, Label, SpinButton, combo_box_entry_new_text, RadioButton, ListStore, \
	ComboBox, CellRendererPixbuf, CellRendererText, CheckButton, Table, EXPAND, FILL, Adjustment
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from common import PLUGIN_PATH, TEXT, MATH

class TableDialog (Dialog):
	def __init__(self):
		Dialog.__init__(self, _("Insert Tabular Object"), None, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(10)
		
		## import data
		self.file_dialog = FileChooserDialog(action=FILE_CHOOSER_ACTION_OPEN, buttons=(STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, RESPONSE_CANCEL))
		self.file_dialog.set_local_only(True)
		self.file_dialog.connect("response", self.__on_file_response)
		#self.file_dialog.connect("file-activated", self.__onFileActivated)
		self.file_button = FileChooserButton(self.file_dialog)
		
		lbl_import = Label()
		lbl_import.set_markup("<b>_Import Data</b>")
		lbl_import.set_alignment(0, .5)
		lbl_import.set_use_underline(True)
		lbl_import.set_mnemonic_widget(self.file_button)
		
		## general
		self.spin_rows = SpinButton(Adjustment(1, 1, 1000, 1), 1)
		self.spin_rows.set_width_chars(7)
		self.spin_rows.set_activates_default(True)
		
		self.spin_cols = SpinButton(Adjustment(2, 1, 1000, 1), 1)
		self.spin_cols.set_width_chars(7)
		self.spin_cols.set_activates_default(True)
		self.spin_cols.connect("value-changed", self.__on_columns_changed)
		
		lbl_rows = Label(_("_Rows:"))
		lbl_rows.set_mnemonic_widget(self.spin_rows)
		lbl_rows.set_use_underline(True)
		lbl_rows.set_alignment(0, .5)
		
		lbl_cols = Label(_("_Columns:"))
		lbl_cols.set_mnemonic_widget(self.spin_cols)
		lbl_cols.set_use_underline(True)
		lbl_cols.set_alignment(0, .5)
		
		self.cmb_layout = combo_box_entry_new_text()
		self.cmb_layout.append_text("l")
		
		lbl_layout = Label(_("Column _Layout:"))
		lbl_layout.set_mnemonic_widget(self.cmb_layout)
		lbl_layout.set_use_underline(True)
		lbl_layout.set_alignment(0, .5)
		
		lbl_general = Label()
		lbl_general.set_markup("<b>General</b>")
		lbl_general.set_alignment(0, .5)
		
		## table
		self.radio_general = RadioButton(None, _("_General (Table)"))
		self.radio_general.connect("toggled", self.__on_radio_toggled)
		
		## matrix
		self.radio_math = RadioButton(self.radio_general, _("_Mathematical (Matrix)"))
		self.radio_math.connect("toggled", self.__on_radio_toggled)
		
		## delimiters
		modelDelims = ListStore(Pixbuf, str)
		modelDelims.append([None, "None"])
		for delim in [("parantheses", _("Parantheses")), ("brackets", _("Brackets")), ("braces", _("Braces")), ("vbars", _("Vertical Bars")), ("dvbars", _("Double Vertical Bars"))]:
			pixbuf = pixbuf_new_from_file("%s/pixmaps/%s.png" % (PLUGIN_PATH, delim[0]))
			modelDelims.append([pixbuf, delim[1]])
		
		self.cmb_delims = ComboBox(modelDelims)
		cellPixbuf = CellRendererPixbuf()
  		self.cmb_delims.pack_start(cellPixbuf, False)
  		self.cmb_delims.add_attribute(cellPixbuf, 'pixbuf', 0)
		cellText = CellRendererText()
  		self.cmb_delims.pack_start(cellText, False)
  		self.cmb_delims.add_attribute(cellText, 'text', 1)
  		self.cmb_delims.set_active(0)
		
		self.lbl_delims = Label(_("_Delimiters:"))
		self.lbl_delims.set_mnemonic_widget(self.cmb_delims)
		self.lbl_delims.set_use_underline(True)
		self.lbl_delims.set_alignment(0, .5)
		
		self.check_ams = CheckButton("Use _AMS environment")
		
		lbl_environ = Label()
		lbl_environ.set_markup("<b>Environment</b>")
		lbl_environ.set_alignment(0, .5)
		
		table = Table()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(lbl_import, 0, 3, 0, 1, EXPAND|FILL, FILL)
		table.attach(self.file_button, 1, 3, 1, 2, EXPAND|FILL, FILL)
		table.attach(lbl_general, 0, 3, 2, 3, EXPAND|FILL, FILL)
		table.attach(lbl_rows, 1, 2, 3, 4, EXPAND|FILL, FILL)
		table.attach(self.spin_rows, 2, 3, 3, 4, EXPAND|FILL, FILL)
		table.attach(lbl_cols, 1, 2, 4, 5, EXPAND|FILL, FILL)
		table.attach(self.spin_cols, 2, 3, 4, 5, EXPAND|FILL, FILL)
		table.attach(lbl_layout, 1, 2, 5, 6, EXPAND|FILL, FILL)
		table.attach(self.cmb_layout, 2, 3, 5, 6, EXPAND|FILL, FILL)
		table.attach(lbl_environ, 0, 3, 6, 7, EXPAND|FILL, FILL)
		table.attach(self.radio_general, 1, 3, 7, 8, EXPAND|FILL, FILL)
		table.attach(self.radio_math, 1, 3, 8, 9, EXPAND|FILL, FILL)
		table.attach(self.lbl_delims, 1, 2, 9, 10, EXPAND|FILL, FILL)
		table.attach(self.cmb_delims, 2, 3, 9, 10, EXPAND|FILL, FILL)
		table.attach(self.check_ams, 1, 3, 10, 11, EXPAND|FILL, FILL)
		
		table.set_col_spacing(0, 15)
		table.set_row_spacing(1, 15)
		table.set_row_spacing(5, 15)
		
		self.vbox.add(table)
		
		self.radio_general.set_active(True)
		self.set_default_response(RESPONSE_OK)
		self.connect("response", self.__on_response)
		
		self.__on_radio_toggled(self.radio_general)
		self.__on_columns_changed(self.spin_cols)
		self.cmb_layout.set_active(0)
	
	def __on_file_response(self, dialog, response):
		if response == RESPONSE_ACCEPT:
			self.imValues = [row.split() for row in open(dialog.get_filename()).readlines() if row[0] != '#' and len(row.split())]
			self.imRows = len(self.imValues)
			if self.imRows:
				self.imCols = len(self.imValues[0])
			else:
				self.imCols = 0
				
			if self.imRows and self.imCols:
				self.spin_rows.set_value(self.imRows)
				self.spin_cols.set_value(self.imCols)
				self.spin_rows.set_sensitive(False)
				self.spin_cols.set_sensitive(False)
	
	def __on_radio_toggled(self, radioButton):
		state = self.radio_math.get_active()
		self.lbl_delims.set_sensitive(state)
		self.cmb_delims.set_sensitive(state)
		self.check_ams.set_sensitive(state)
	
	def __on_columns_changed(self, spinButton):
		""" suggest layouts that fit the number of columns """
		cols = spinButton.get_value_as_int()
		self.cmb_layout.get_model().clear()
		for layout in ["l" * cols, "|".join("l" * cols), "@{\\ \\vline\\ }".join("l" * cols)]:
			self.cmb_layout.append_text(layout)	
	
	def __build_source(self):
		if self.imValues:
			## create a grid from the imported values
			rows = ["\t" + " & ".join(row) for row in self.imValues]
			grid = " \\\\\n".join(rows)
		else:
			## create a grid of placeholders
			r = "\t" + " & ".join("x" * self.spin_cols.get_value_as_int())
			grid = " \\\\\n".join([r] * self.spin_rows.get_value_as_int())
		
		## wrap it into an environment
		if self.radio_general.get_active():
			self.source = "\\begin{tabular}{%s}\n%s\n\\end{tabular}" % (self.cmb_layout.get_active_text(), grid)
		else:
			delimIndex = self.cmb_delims.get_active()
			if self.check_ams.get_active():
				environment = ["matrix", "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix"][delimIndex]
				self.source = "\\begin{%s}\n%s\n\\end{%s}" % (environment, grid, environment)
				self.package = "amsmath"
			else:
				lDelim = ["", r"\left(", r"\left[", r"\left\lbrace", r"\left\vert", r"\left\Vert"][delimIndex]
				rDelim = ["", r"\right(", r"\right[", r"\right\rbrace", r"\right\vert", r"\right\Vert"][delimIndex]
				self.source = "%s\\begin{array}{%s}\n%s\n\\end{array}%s" % (lDelim, self.cmb_layout.get_active_text(), grid, rDelim)
	
	def __on_response(self, dialog, response):
		if response == RESPONSE_OK:
			self.__build_source()
	
	def get_source(self):
		return self.source
		
	def get_package(self):
		return self.package
	
	def run(self, environment):
		## reset ui
		self.file_dialog.unselect_all()
		self.spin_rows.set_sensitive(True)
		self.spin_cols.set_sensitive(True)
		self.imValues = None
		self.radio_general.set_active(environment == TEXT)
		self.radio_math.set_active(environment == MATH)
		
		self.show_all()
		return Dialog.run(self)
		
