# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gedit
from xml.parsers.expat import ExpatError

from PluginInstance import PluginInstance
from Settings import Settings, PLUGIN_PATH, str_bool
from Logger import log

class LaTeXPlugin(gedit.Plugin):
	def __init__(self):
		gedit.Plugin.__init__(self)
		self.TAG = "LaTeXPluginInstance"
		
		self.__settings = None
		f = "%s/settings.xml" % PLUGIN_PATH
		try:
			self.__settings = Settings(f)
			
			# migrate from GConf
			if not str_bool(self.__settings["migrated"]):
				log(self, "Migrating from GConf...")
				
				from Settings import GConfMigrator
				
				migrator = GConfMigrator(self.__settings)
				migrator.run()
				
				self.__settings["migrated"] = True
				
		except ExpatError, message:
			log(self, "XML parser error in %s: %s" % (f, message))
		
	def __getInstance(self, window):
		return window.get_data(self.TAG)
	
	def __setInstance(self, window, instance):
		window.set_data(self.TAG, instance)
	
	def activate(self, window):
		"""
		Called when 
		 * a new gedit window is created
		 * the plugin is activated in the plugin settings dialog
		 * gedit is started with the plugin loaded
		"""
		if self.__settings:
			self.__setInstance(window, PluginInstance(self, window, self.__settings))
	
	def deactivate(self, window):
		"""
		Called when the plugin is deactivated by the user
		"""
		self.__getInstance(window).deactivate()
		self.__setInstance(window, None)
	
	def update_ui(self, window):
		"""
		Called when the window is updated (tab change, menu update etc)
		"""
		self.__getInstance(window).update()
	
	def create_configure_dialog(self):
		"""
		Called when the plugin is to be configured (must return a gtk.Window)
		"""
		from ConfigureDialog import ConfigureDialog
		
		if self.__settings:
			return ConfigureDialog(self.__settings)
		
