# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from os import popen
from os.path import basename
from re import compile, M

from Message import Message, SEVERITY_ERROR, SEVERITY_WARNING

class RubberProcessor(object):
	"""
	"""
	#
	# TODO: falls rubber nicht installiert ist, entspr. Message generieren
	# vgl. Builder.__rubber_installed()
	#
	# TODO: die Messages insgesamt verbessern: weniger pro Zeile
	#
	def __init__(self):
		self.__pattern = None
	
	def __get_pattern(self):
		if not self.__pattern:
			self.__pattern = compile(r"(?P<f>[a-zA-Z0-9./_-]+):(?P<l>[0-9]+):(?P<t>.*?)$", M)
		return self.__pattern
	
	def get_messages(self, input_filename, condition):
		"""
		Retrieve messages for a document that has been built with rubber
		"""
		if condition:
			# rubber failed, return errors warnings
			output = popen("rubber-info --errors %s 2>&1" % input_filename).read()
			severity = SEVERITY_ERROR
		else:
			# rubber succeeded, return only warnings
			output = popen("rubber-info --warnings %s 2>&1" % input_filename).read()
			severity = SEVERITY_WARNING
		
		return [Message(m.group("f"), int(m.group("l")) - 1, m.group("t"), severity) for m in self.__get_pattern().finditer(output)]